import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/scheduler.dart';

/// 控制折叠的widget
///
/// PutAwayDirection: 折叠的方向
/// rtl 从右向左折叠
/// btt 从下到上折叠
enum PutAwayDirection { rtl, btt }

class PutAway extends SingleChildRenderObjectWidget {
  final PutAwayDirection direction;
  final bool isPutAway;

  const PutAway({
    Key? key,
    required this.direction,
    this.isPutAway = false,
    required Widget child,
  }) : super(key: key, child: child);

  @override
  _RenderPutAway createRenderObject(BuildContext context) {
    return _RenderPutAway(direction, isPutAway);
  }

  @override
  void updateRenderObject(BuildContext context, _RenderPutAway renderObject) {
    renderObject
      ..direction = direction
      ..isPutAway = isPutAway;
  }
}

class _RenderPutAway extends RenderBox
    with RenderObjectWithChildMixin<RenderBox>
    implements TickerProvider {
  _RenderPutAway(this._direction, this._isPutAway);

  PutAwayDirection _direction;

  set direction(PutAwayDirection value) {
    if (_direction != value) {
      _direction = value;
      markNeedsLayout();
    }
  }

  bool _isPutAway = false;

  set isPutAway(bool value) {
    if (_isPutAway != value) {
      _isPutAway = value;
      if (_isPutAway) {
        _controller.forward();
      } else {
        _controller.reverse();
      }
    }
  }

  late AnimationController _controller;
  late Animation _animation;

  @override
  Ticker createTicker(TickerCallback onTick) => Ticker(onTick);

  @override
  void attach(covariant PipelineOwner owner) {
    super.attach(owner);
    _controller = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 200),
    );
    _animation = CurvedAnimation(
      parent: _controller,
      curve: Curves.easeIn,
      reverseCurve: Curves.easeOut,
    );
    _controller.addListener(markNeedsLayout);
  }

  Size _childSize = Size.zero;

  @override
  void performLayout() {
    child!.layout(constraints, parentUsesSize: true);
    _childSize = child!.size;
    switch (_direction) {
      case PutAwayDirection.rtl:
        size = Size(
          child!.size.width * (1 - _animation.value),
          child!.size.height,
        );
        break;
      case PutAwayDirection.btt:
        size = Size(
          child!.size.width,
          child!.size.height * (1 - _animation.value),
        );
        break;
      default:
        size = child!.size;
    }
  }

  @override
  void detach() {
    _controller.stop();
    _controller.removeListener(markNeedsLayout);
    _controller.dispose();
    super.detach();
  }

  @override
  bool hitTestSelf(Offset position) => true;

  @override
  bool hitTestChildren(BoxHitTestResult result, {required Offset position}) {
    return child!.hitTest(result, position: position);
  }

  @override
  void paint(PaintingContext context, Offset offset) {
    super.paint(context, offset);
    if (_animation.value == 0.0) {
      context.paintChild(child!, offset);
      return;
    }
    if (_animation.value == 1.0) return;
    switch (_direction) {
      case PutAwayDirection.rtl:
        layer = context.pushClipRect(
          needsCompositing,
          offset,
          Offset.zero & size,
          child!.paint,
          oldLayer: layer as ClipRectLayer?,
        );
        break;
      case PutAwayDirection.btt:
        layer = context.pushClipRect(
          needsCompositing,
          offset,
          Offset.zero & size,
          child!.paint,
          oldLayer: layer as ClipRectLayer?,
        );
        break;
    }
  }
}
